@@ -13,7 +13,7 @@ from resolvelib.resolvers import Criterion
 from pdm.exceptions import CandidateNotFound, InvalidPyVersion, RequirementError
 from pdm.models.candidates import Candidate
 from pdm.models.repositories import LockedRepository
-from pdm.models.requirements import FileRequirement, Requirement, parse_requirement, strip_extras
+from pdm.models.requirements import FileRequirement, Requirement, VcsRequirement, parse_requirement, strip_extras
 from pdm.models.specifiers import PySpecSet
 from pdm.resolver.python import PythonCandidate, PythonRequirement, find_python_matches, is_python_satisfied_by
 from pdm.termui import logger
@@ -90,7 +90,7 @@ class BaseProvider(AbstractProvider[Requirement, Candidate, str]):
         editable = requirement.editable
         is_named = requirement.is_named
         is_pinned = requirement.is_pinned
-        is_prerelease = bool(requirement.prerelease) and bool(requirement.specifier.prereleases)
+        is_prerelease = bool(requirement.prerelease) or bool(requirement.specifier.prereleases)
         specifier_parts = len(requirement.specifier)
         return (not editable, is_named, not is_pinned, not is_prerelease, -specifier_parts)
 
@@ -209,9 +209,9 @@ class BaseProvider(AbstractProvider[Requirement, Candidate, str]):
                     # If there are wheels for the given version, we should only return wheels
                     # to avoid build steps.
                     if collected_wheels:
-                        yield from collected_wheels
-                    else:
-                        yield from collected_others
+                        yield collected_wheels[0]
+                    elif collected_others:
+                        yield collected_others[0]
                     current_version = candidate.version
                     collected_wheels.clear()
                     collected_others.clear()
@@ -220,9 +220,9 @@ class BaseProvider(AbstractProvider[Requirement, Candidate, str]):
                 else:
                     collected_others.append(candidate)
             if collected_wheels:
-                yield from collected_wheels
-            else:
-                yield from collected_others
+                yield collected_wheels[0]
+            elif collected_others:
+                yield collected_others[0]
 
     def find_matches(
         self,
@@ -415,7 +415,10 @@ class ReusePinProvider(BaseProvider):
         return matches_gen
 
     def _get_dependencies_from_repository(self, candidate: Candidate) -> tuple[list[Requirement], PySpecSet, str]:
-        if self.locked_repository is not None:
+        is_stable_metadata = candidate.req.is_named or (
+            isinstance(candidate.req, VcsRequirement) and candidate.req.revision
+        )
+        if self.locked_repository is not None and is_stable_metadata:
             try:
                 return self.locked_repository.get_dependencies(candidate)
             except CandidateNotFound:
@@ -479,4 +482,4 @@ class ReuseInstalledProvider(ReusePinProvider):
             return super().iter_reuse_candidates(identifier, requirement)
         else:
             dist = self.installed[key]
-            return [Candidate(requirement, name=dist.metadata["Name"], version=dist.metadata["Version"])]
+            return [Candidate(requirement, installed=dist)]
